<?php
/*******************************************************************************
* SMF Arcade 2.0.18 (http://www.smfarcade.info)                                *
* Copyright (C) 2004-2009  Niko Pahajoki (http://www.madjoki.com)              *
* This program is free software; you can redistribute it and/or modify         *
* it under the terms of the GNU General Public License as published by         *
* the Free Software Foundation; either version 2 of the License, or            *
* (at your option) any later version.                                          *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program; if not, write to the Free Software                  *
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA *
********************************************************************************
* Arcade.php                                                                   *
********************************************************************************/

if (!defined('SMF'))
	die('Hacking attempt...');

/*	This file handles Arcade and loads required files.

	void Arcade()
		- Initialzes Arcade
		- Loads langauge files
		- Loads Template
		- Loads Arcade

	void ArcadeLoad([mode = normal])
		- Initializes Arcade

	string PermissionQuery()
		- Returns permission query to use in WHERE
*/

function Arcade()
{
	global $scripturl, $txt, $context, $sourcedir, $ID_MEMBER, $modSettings;

	// Do we have permission?
	isAllowedTo('arcade_view');

	// Information for actions (file, function, [permission])
	$subActions = array(
		// ArcadeList.php
		'rate' => array('ArcadeList.php', 'ArcadeRate', 'arcade_rate'),
		'list' => array('ArcadeList.php', 'ArcadeList'),
		'favorite' => array('ArcadeList.php', 'ArcadeFavorite', 'arcade_favorite'),
		'search' => array('ArcadeList.php', 'ArcadeSearch'),
		// ArcadePlay.php
		'play' => array('ArcadePlay.php', 'ArcadePlay', 'arcade_play'),
		'submit' => array('ArcadePlay.php', 'ArcadeSubmit'),
		'highscore' => array('ArcadePlay.php', 'ArcadeHighscore'),
		'comment' => array('ArcadePlay.php', 'ArcadeComment'),
		// ArcadeStats.php
		'stats' => array('ArcadeStats.php', 'ArcadeStats'),
		// IBP Arcade
		'ibpverify' => array('ArcadePlay.php', 'ArcadeVerifyIBP'),
		'ibpsubmit' => array('ArcadePlay.php', 'ArcadeSubmitIBP'),
	);

	// Fix for broken games which doesn't send sa/do=submit
	if (isset($_POST['game']) && isset($_POST['score']) && !isset($_REQUEST['sa']))
		$_REQUEST['sa'] = 'submit';

	// Short urls like index.php?game=1 or index.php/game,1.html
	elseif (isset($_REQUEST['game']) && is_numeric($_REQUEST['game']) && !isset($_REQUEST['sa']))
		$_REQUEST['sa'] = 'play';

	$_REQUEST['sa'] = isset($_REQUEST['sa']) && isset($subActions[$_REQUEST['sa']]) ? $_REQUEST['sa'] : 'list';

	// Load Arcade
   	ArcadeLoad('normal');

	if (!in_array($_REQUEST['sa'], array('highscore', 'comment')) && isset($_SESSION['arcade']['highscore']))
   		unset($_SESSION['arcade']['highscore']);

   	// Check permission if needed
   	if (isset($actions[$_REQUEST['sa']][2]))
   		isAllowedTo($subActions[$_REQUEST['sa']][2]);

	require_once($sourcedir . '/' . $subActions[$_REQUEST['sa']][0]);
	$subActions[$_REQUEST['sa']][1]();
}

function ArcadeLoad($mode = 'normal', $index = '')
{
	global $db_prefix, $scripturl, $txt, $modSettings, $context, $settings, $sourcedir, $user_info, $arcade_version, $ID_MEMBER, $user_profile, $boarddir;

	require_once($sourcedir . '/Subs-Arcade.php');

	$context['arcade'] = array();

	// Version information
	$arcade_version = '2.0.18';

	$user_info['query_see_game'] = PermissionQuery();

	loadLanguage('Arcade');

	// Arcade javascript
	$context['html_headers'] .= '
	<script language="JavaScript" type="text/javascript" src="' . $settings['default_theme_url'] . '/arcade.js"></script>';

	// Normal mode
	if ($mode == 'normal')
	{
		// Is Arcade enabled?
		if (empty($modSettings['arcadeEnabled']))
			fatal_lang_error('arcade_disabled');

		loadTemplate('Arcade');
		loadTemplate('ArcadeMenu');

		// Title
		$context['page_title'] = $txt['arcade'];

		// Add Arcade to link tree
		$context['linktree'][] = array(
			'url' => $scripturl . '?action=arcade',
			'name' => $txt['arcade'],
		);
		$context['arcade_tabs'] = array(
			'title' =>  $txt['arcade'],
		);

		// Tabs to show in every Arcade page
		$context['arcade_tabs']['tabs'] = array(
			array(
				'href' => $scripturl . '?action=arcade',
				'title' => $txt['arcade'],
				'is_selected' => in_array($_REQUEST['sa'], array('play', 'list', 'highscore', 'submit')),
			),
			array(
				'href' => $scripturl . '?action=arcade;sa=stats',
				'title' => $txt['arcade_stats'],
				'is_selected' => in_array($_REQUEST['sa'], array('stats')),
			)
		);

		// What I can do?
		$context['arcade']['can_play'] = allowedTo('arcade_play');
		$context['arcade']['can_favorite'] = allowedTo('arcade_favorite');
		$context['arcade']['can_rate'] = allowedTo('arcade_rate');
		$context['arcade']['can_submit'] = allowedTo('arcade_submit');
		$context['arcade']['can_comment_own'] = allowedTo('arcade_comment_own');
		$context['arcade']['can_comment_any'] = allowedTo('arcade_comment_any');
		$context['arcade']['can_admin'] = allowedTo('arcade_admin');

		// Or can I (do I have enought posts etc.)
		if (!empty($modSettings['arcadePostPermission']) && !$context['arcade']['can_admin'] && $context['arcade']['can_play'])
		{
			if ($user_info['is_guest'])
			{
				$context['arcade']['can_play'] = false;

				if (!empty($modSettings['arcadePostsPlay']) && !empty($modSettings['arcadePostsPlayPerDay']))
					$context['arcade']['notice'] = sprintf($txt['arcade_notice_post_both'], $modSettings['arcadePostsPlay'], $modSettings['arcadePostsPlayPerDay']);

				elseif (!empty($modSettings['arcadePostsPlay']))
					$context['arcade']['notice'] = sprintf($txt['arcade_notice_post_cumulative'], $modSettings['arcadePostsPlay']);

				elseif (!empty($modSettings['arcadePostsPlayPerDay']))
					$context['arcade']['notice'] = sprintf($txt['arcade_notice_post_perday'], $modSettings['arcadePostsPlayPerDay']);

			}
			else
			{
				loadMemberData($ID_MEMBER, false, 'minimal');

				$days = ceil(time() - $user_profile[$ID_MEMBER]['dateRegistered'] / 86400);

				// Who didn't syncronise clock?
				if ($days < 1)
					$days = 1;

				$post = $user_profile[$ID_MEMBER]['posts'];

				$from = time() - 86400;

				$result = db_query("
					SELECT COUNT(*)
					FROM {$db_prefix}messages AS m
						LEFT JOIN {$db_prefix}boards AS b ON (m.ID_BOARD = b.ID_BOARD)
					WHERE b.countPosts != 1
						AND m.ID_MEMBER = $ID_MEMBER
						AND m.posterTime >= $from", __FILE__, __LINE__);

				list ($postPerDay) = mysql_fetch_row($result);
				mysql_free_result($result);

				if (!empty($modSettings['arcadePostsPlay']))
					$post2 = $post >= $modSettings['arcadePostsPlay'];
				else
					$post2 = true;

				if (!empty($modSettings['arcadePostsPlayPerDay']))
					$postPerDay2 = $postPerDay >= $modSettings['arcadePostsPlayPerDay'];
				else
					$postPerDay2 = true;

				$context['arcade']['can_play'] = $post2 && $postPerDay2;

				// Should we display notice, and what kind of notice?
				if (!$post2 || !$postPerDay2)
				{
					$p1 = !$post2 ?	'<span style="color: red">' . $modSettings['arcadePostsPlay'] . '</span>' : $modSettings['arcadePostsPlay'];
					$p2 = !$postPerDay2 ? '<span style="color: red">' . $modSettings['arcadePostsPlayPerDay'] . '</span>' :  $modSettings['arcadePostsPlayPerDay'];

					if (!empty($modSettings['arcadePostsPlay']) && !empty($modSettings['arcadePostsPlayPerDay']))
						$context['arcade']['notice'] = sprintf($txt['arcade_notice_post_both'], $p1, $p2);

					elseif (!empty($modSettings['arcadePostsPlay']))
						$context['arcade']['notice'] = sprintf($txt['arcade_notice_post_cumulative'], $p1);

					elseif (!empty($modSettings['arcadePostsPlayPerDay']))
						$context['arcade']['notice'] = sprintf($txt['arcade_notice_post_perday'], $p2);
				}
			}
		}

		$context['arcade']['stats'] = array();

		// How many games?
		$result = db_query("
			SELECT count(ID_GAME) AS games
			FROM {$db_prefix}arcade_games
			WHERE enabled = 1", __FILE__, __LINE__);
		$context['arcade']['stats'] += mysql_fetch_assoc($result);

		mysql_free_result($result);

		require_once($sourcedir . '/ArcadeStats.php');

		$context['arcade']['best_player'] = ArcadeStats_BestPlayers(1);
		$context['arcade']['longest_champion'] = ArcadeStats_LongestChampions(1, 'current');
		$context['arcade']['most_played'] = ArcadeStats_MostPlayed(1);

		// Menu
		$context['arcade']['menu'] = array(
			array(
				'data' => ArcadeGameOfDay(0),
				'template' => 'ArcadeMenu_GameOfDay'
			),
			array(
				'data' => array(),
				'template' => 'ArcadeMenu_QuickSearch',
			),
		);

		$context['arcade']['latest_scores'] = ArcadeLatestScores(5, 0);

		if (!isset($_REQUEST['xml']))
		{
			$context['template_layers'][] = 'Arcade';
			$context['template_layers'][] = 'ArcadeMenu';
		}
	}
	// Admin mode
	elseif ($mode == 'admin')
	{
		loadTemplate('ArcadeAdmin');
		loadLanguage('ArcadeAdmin');

		if (file_exists($boarddir . '/installArcade.php') && @!unlink($boarddir . '/installArcade.php'))
			fatal_lang_error('arcade_arcade_installer_not_removed');

		if (!empty($index))
			adminIndex($index);

		$context['template_layers'][] = 'ArcadeAdmin';
		$context['page_title'] = $txt['arcade_admin_title'];
	}
}

function PermissionQuery()
{
	global $scripturl, $txt, $modSettings, $context, $settings, $sourcedir, $user_info;

	// Build permission query
	if (!isset($modSettings['arcadePermissionMode']))
		$modSettings['arcadePermissionMode'] = 1;

	if ($modSettings['arcadePermissionMode'] >= 2)
	{
		// Can see game?
		if ($user_info['is_guest'])
			$see_game = 'FIND_IN_SET(-1, game.memberGroups)';

		// Administrators can see all games.
		elseif ($user_info['is_admin'])
			$see_game = '1';
		// Registered user.... just the groups in $user_info['groups'].
		else
			$see_game = '(FIND_IN_SET(' . implode(', game.memberGroups) OR FIND_IN_SET(', $user_info['groups']) . ', game.memberGroups))';
	}

	if ($modSettings['arcadePermissionMode'] == 1 || $modSettings['arcadePermissionMode'] >= 3)
	{
		// Can see category?
		if ($user_info['is_guest'])
			$see_category = 'FIND_IN_SET(-1, category.memberGroups)';

		// Administrators can see all games.
		elseif ($user_info['is_admin'])
			$see_category = '1';
		// Registered user.... just the groups in $user_info['groups'].
		else
			$see_category = '(FIND_IN_SET(' . implode(', category.memberGroups) OR FIND_IN_SET(', $user_info['groups']) . ', category.memberGroups))';
	}

	// Build final query
	if ($modSettings['arcadePermissionMode'] == 0) // No game/category permissions used
		return '1';

	elseif ($modSettings['arcadePermissionMode'] == 1) // Only category used
		return $see_category;

	elseif ($modSettings['arcadePermissionMode'] == 2) // Only category used
		return $see_game;

	elseif ($modSettings['arcadePermissionMode'] == 3) // Required to have permssion to game and category
		return "($see_category AND $see_game)";

	elseif ($modSettings['arcadePermissionMode'] == 4) // Required to have permssion to game OR category
		return "($see_category OR $see_game)";

	else // Default
		return $see_category;
}

?>