<?php
/*******************************************************************************
* SMF Arcade 2.0.12 (http://www.smfarcade.info)                                *
* Copyright (C) 2004-2007  Niko Pahajoki (http://www.madjoki.com)              *
* This program is free software; you can redistribute it and/or modify         *
* it under the terms of the GNU General Public License as published by         *
* the Free Software Foundation; either version 2 of the License, or            *
* (at your option) any later version.                                          *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program; if not, write to the Free Software                  *
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA *
********************************************************************************
* ArcadeStats.php                                                              *
********************************************************************************/

if (!defined('SMF'))
	die('Hacking attempt...');

/*
	void ArcadeStats()
		- ...

	array ArcadeStats_BestPlayers([count = 10])
		- ...

	array ArcadeStats_LongestChampions([count = 10], [time])
		- ...

	array ArcadeStats_MostActive([count = 10], [time])
		- ...

	array ArcadeStats_MostPlayed([count = 10], [time])
		- ...

	array ArcadeStats_Rating([count = 10])
		- ...

	string format_time_ago(timestamp)
		- Formats time to xxx ago

*/

function ArcadeStats()
{
	global $txt, $context;

	$context['sub_template'] = 'arcade_statistics';
	$context['page_title'] = $txt['arcade_stats_title'];

	// Load data using functions
	$context['arcade']['statistics']['play'] = ArcadeStats_MostPlayed();
	$context['arcade']['statistics']['active'] = ArcadeStats_MostActive();
	$context['arcade']['statistics']['rating'] = ArcadeStats_Rating();
	$context['arcade']['statistics']['champions'] = ArcadeStats_BestPlayers();
	$context['arcade']['statistics']['longest'] = ArcadeStats_LongestChampions();

}

function ArcadeStats_MostPlayed($count = 10)
{
	// Returns most playd games
	global $db_prefix, $scripturl;

	$results = db_query("
		SELECT game.ID_GAME, game.gameName, game.gameRating, game.numPlays
		FROM {$db_prefix}arcade_games AS game
		ORDER BY game.numPlays DESC
		LIMIT $count", __FILE__, __LINE__);

	$top = array();
	$max = -1;

	while ($score = mysql_fetch_assoc($results))
	{
		if ($max == -1)
			$max = $score['numPlays'];
		if ($max == 0)
			return false; // No one has played games yet0

		$top[] = array(
			'id' => $score['ID_GAME'],
			'name' => $score['gameName'],
			'link' => '<a href="' . $scripturl . '?action=arcade;sa=play;game=' . $score['ID_GAME'] . '">' .  $score['gameName'] . '</a>',
			'rating' => $score['gameRating'],
			'plays' => $score['numPlays'],
			'precent' => ($score['numPlays'] / $max) * 100,
		);
	}

	if (count($top) == 0)
		return false;
	elseif ($count > 1)
		return $top;
	else
		return $top[0];
}

function ArcadeStats_Rating($count = 10)
{
	global $db_prefix, $scripturl;

	$results = db_query("
		SELECT game.ID_GAME, game.gameName, game.gameRating, game.numPlays
		FROM {$db_prefix}arcade_games AS game
		WHERE gameRating > 0
		ORDER BY game.gameRating DESC
		LIMIT $count", __FILE__, __LINE__);

	$top = array();
	$max = -1;

	while ($score = mysql_fetch_assoc($results))
	{
		if ($max == -1)
			$max = $score['gameRating'];

		$top[] = array(
			'id' => $score['ID_GAME'],
			'name' => $score['gameName'],
			'link' => '<a href="' . $scripturl . '?action=arcade;sa=play;game=' . $score['ID_GAME'] . '">' .  $score['gameName'] . '</a>',
			'rating' => $score['gameRating'],
			'plays' => $score['numPlays'],
			'precent' => ($score['gameRating'] / $max) * 100,
		);
	}

	if (count($top) == 0)
		return false;
	elseif ($count > 1)
		return $top;
	else
		return $top[0];
}

function ArcadeStats_BestPlayers($count = 10)
{
	// Returns best players by count of champions
	global $db_prefix, $scripturl, $txt;

	$results = db_query("
		SELECT count(*) AS champions, IFNULL(mem.ID_MEMBER, 0) AS ID_MEMBER, IFNULL(mem.realName, '') AS realName
		FROM {$db_prefix}arcade_games AS game
			LEFT JOIN {$db_prefix}members AS mem ON (mem.ID_MEMBER = game.ID_MEMBER_CHAMPION)
		WHERE ID_CHAMPION_SCORE > 0
		GROUP BY game.ID_MEMBER_CHAMPION
		ORDER BY champions DESC
		LIMIT $count", __FILE__, __LINE__);

	$top = array();
	$max = -1;

	while ($score = mysql_fetch_assoc($results))
	{
		if ($max == -1)
			$max = $score['champions'];

		$top[] = array(
			'name' => $score['realName'],
			'link' => !empty($score['realName']) ? '<a href="' . $scripturl . '?action=profile;u=' . $score['ID_MEMBER'] . '">' .  $score['realName'] . '</a>' : $txt['arcade_guest'],
			'champions' => $score['champions'],
			'precent' => ($score['champions'] / $max) * 100,
		);
	}

	if (count($top) == 0)
		return false;
	elseif ($count > 1)
		return $top;
	else
		return $top[0];
}

function ArcadeStats_MostActive($count = 10, $time = -1)
{
	// Returns most active players
	global $db_prefix, $scripturl, $txt;

	$results = db_query("
		SELECT count(*) AS scores, IFNULL(mem.ID_MEMBER, 0) AS ID_MEMBER, IFNULL(mem.realName, '') AS realName
		FROM {$db_prefix}arcade_scores AS score
			LEFT JOIN {$db_prefix}members AS mem ON (mem.ID_MEMBER = score.ID_MEMBER)
		GROUP BY score.ID_MEMBER
		ORDER BY scores DESC
		LIMIT $count", __FILE__, __LINE__);

	$top = array();
	$max = -1;

	while ($score = mysql_fetch_assoc($results))
	{
		if ($max == -1)
			$max = $score['scores'];

		$top[] = array(
			'name' => $score['realName'],
			'link' => !empty($score['realName']) ? '<a href="' . $scripturl . '?action=profile;u=' . $score['ID_MEMBER'] . '">' .  $score['realName'] . '</a>' : $txt['arcade_guest'],
			'scores' => $score['scores'],
			'precent' => ($score['scores'] / $max) * 100,
		);
	}

	if (count($top) == 0)
		return false;
	elseif ($count > 1)
		return $top;
	else
		return $top[0];
}

function ArcadeStats_LongestChampions($count = 10, $time = - 1)
{
	global $db_prefix, $scripturl, $txt;

	$where = '';

	if ($time == 'current')
		$where = 'AND championTo = 0';
	elseif ($time != -1)
		$where = "AND championFrom > $time";

	$results = db_query("
		SELECT game.ID_GAME, game.gameName,
			IF(championFrom > 0, (IF(championTo = 0, UNIX_TIMESTAMP(), championTo) - championFrom), 0) AS championDuration,
			IFNULL(mem.ID_MEMBER, 0) AS ID_MEMBER, IFNULL(mem.realName, '') AS realName, IF(championTo = 0, 1, 0) AS current
		FROM ({$db_prefix}arcade_scores AS score, {$db_prefix}arcade_games AS game)
			LEFT JOIN {$db_prefix}members AS mem ON (mem.ID_MEMBER = score.ID_MEMBER)
		WHERE game.ID_GAME = score.ID_GAME
			$where
		HAVING championDuration > 0
		ORDER BY IF(championFrom > 0, (IF(championTo = 0, UNIX_TIMESTAMP(), championTo) - championFrom), 0) DESC
		LIMIT $count", __FILE__, __LINE__);

	$top = array();
	$max = -1;

	while ($score = mysql_fetch_assoc($results))
	{
		if ($max == -1)
			$max = $score['championDuration'];

		$top[] = array(
			'game_name' => $score['gameName'],
			'game_link' => '<a href="' . $scripturl . '?action=arcade;sa=play;game=' . $score['ID_GAME'] . '">' .  $score['gameName'] . '</a>',
			'member_name' => $score['realName'],
			'member_link' => !empty($score['realName']) ? '<a href="' . $scripturl . '?action=profile;u=' . $score['ID_MEMBER'] . '">' .  $score['realName'] . '</a>' : $txt['arcade_guest'],
			'duration' => format_time_ago($score['championDuration']),
			'precent' => ($score['championDuration'] / $max) * 100,
			'current' => $score['current'] == 1,
		);
	}

	if (count($top) == 0)
		return false;
	elseif ($count > 1)
		return $top;
	else
		return $top[0];
}

function format_time_ago($timestamp)
{
	global $txt;
	// Returns formated string

	$yksikot = array(
		array(604800, $txt['arcade_weeks']), // Seconds in week
		array(86400, $txt['arcade_days']), // Seconds in day
		array(3600, $txt['arcade_hours']), // Seconds in hour
		array(60, $txt['arcade_mins']), // Seconds in minute
	);

	if ($timestamp < 60)
		return $txt['arcade_under_minute_ago'];

	$text = '';

	foreach( $yksikot as $t )
	{
		$tassa = floor($timestamp / $t[0]);

		if ( $tassa > 1 )
		{
			$text .= $tassa . ' '. $t[1];
			$text .= ' ';

			$timestamp = $timestamp - ($t[0] * $tassa);
		}
	}

	return trim($text);
}
?>