<?php
/*******************************************************************************
* SMF Arcade 2.0.16 (http://www.smfarcade.info)                                *
* Copyright (C) 2004-2009  Niko Pahajoki (http://www.madjoki.com)              *
* This program is free software; you can redistribute it and/or modify         *
* it under the terms of the GNU General Public License as published by         *
* the Free Software Foundation; either version 2 of the License, or            *
* (at your option) any later version.                                          *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program; if not, write to the Free Software                  *
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA *
********************************************************************************
* Subs-Arcade.php                                                              *
********************************************************************************/

if (!defined('SMF'))
	die('Hacking attempt...');

/*	This file contains all functions used by arcade

	void ArcadeGameOfDay([day = 0])
		- Returns Game of Day

	array NewGameOfDay()
		- Generates new game of day, calls function customGameOfDay if it is defined

	array ArcadeLatestScores([count = 5], [start = 0])
		- Returns latest scores

	array ArcadeGameInfo([ID_GAME], [internalName], [mode = normal])
		- Returns info of game
		- Either by ID or internalName
		- Mode is not used yet

	string ArcadeFlashHtml(url, file, width, height, bgcolor)
		- Constructs HTML-code for flash games

	array ArcadeFixPositions(ID_GAME)
		- Fixes positions of game
		- Returns array of errors found

	array ArcadeSaveScore(ID_GAME, score, startTime, endTime, scoreStatus, [reverse = false])
		- Saves score into database, handling everything needed

	boolean ArcadeSendPM(type, game, [extra])
		- Sends PMs

	array prepareCategories()
		- Gets list of categories

	void FixCategores
		- Fixes games with invalid category

	void prepareMemberGroups()
		- prepares list of membergroups

	boolean UpdateGame()
		- ...

	array BuildGameArray(game)
	array BuildGameArrayAdmin(game)
		- Builds array for easy to use

*/

// Return game of day
function ArcadeGameOfDay($day = 0)
{
	global $db_prefix;

	// Return 'Game of day'

	if ($day == 0)
		$day = date('ymd');

	$game = db_query("
	  SELECT ID_GAME
	  FROM {$db_prefix}arcade_gameofday
	  WHERE day = $day", __FILE__, __LINE__);

	if (mysql_num_rows($game) == 0)
		return NewGameOfDay();
	else
	{
	  $id_game = mysql_fetch_assoc($game);
	  return ArcadeGameInfo((int) $id_game['ID_GAME']);
	}
}

// Generates new game of day
function NewGameOfDay()
{
	global $db_prefix;

	if (!function_exists('customGameOfDay'))
		$game = ArcadeGameInfo();

	else
		// Theres custom function
		$game = customGameOfDay();

	if ($game === false)
		return false;

	$day = date('ymd');

	db_query("
		INSERT INTO {$db_prefix}arcade_gameofday
		SET day = $day, ID_GAME = $game[id]", __FILE__, __LINE__);

	return $game;

}

// Return Latest scores
function ArcadeLatestScores($count = 5, $start = 0)
{
	global $scripturl, $txt, $db_prefix;

	$scores = db_query("
		SELECT game.ID_GAME, game.gameName, score.score, score.position,
			IFNULL(mem.ID_MEMBER, 0) AS ID_MEMBER, IFNULL(mem.realName, '') AS realName, score.endTime
		FROM ({$db_prefix}arcade_scores AS score, {$db_prefix}arcade_games AS game)
			LEFT JOIN {$db_prefix}members AS mem ON (mem.ID_MEMBER = score.ID_MEMBER)
		WHERE game.ID_GAME = score.ID_GAME
		ORDER BY endTime DESC
		LIMIT $start,$count",__FILE__,__LINE__);

	$data = array();

	while ($score = mysql_fetch_array($scores))
		$data[] = array(
			'game_id' => $score['ID_GAME'],
			'name' => $score['gameName'],
			'score' => comma_format($score['score']),
			'id' => $score['ID_MEMBER'],
			'member' => $score['realName'],
			'memberLink' => $score['realName'] != '' ? '<a href="' . $scripturl . '?action=profile;u=' . $score['ID_MEMBER'] . '">' . $score['realName'] . '</a>' : $txt['arcade_guest'],
			'time' => timeformat($score['endTime']),
		);

	return $data;
}

// Fetches game info
function ArcadeGameInfo($ID_GAME = 0, $internalName = '', $admin = false)
{
	global $scripturl, $txt, $db_prefix, $user_info, $ID_MEMBER;

	// Which mode to use
	if (empty($ID_GAME) && empty($internalName)) // Random
		$sql = 'ORDER BY RAND()';

	elseif (!empty($ID_GAME) && empty($internalName)) // By ID
		$sql = 'AND game.ID_GAME = ' . (int) $ID_GAME;

	elseif (!empty($internalName)) // By internalName
		$sql = 'AND game.internalName = \'' . $internalName . '\'';

	else
		return false;

	if (!$admin)
		$enabled = 'enabled = 1';

	else
		$enabled = '1';

	if ($admin && empty($user_info['query_see_game']))
		$user_info['query_see_game'] = '1';

	// Query with all needed data
	$query = "
		SELECT game.ID_GAME, game.gameName, game.description, game.gameRating, game.numPlays, game.gameFile,
			game.gameDirectory, game.gameType, game.internalName, game.gameWidth, game.gameHeight,
			game.gameBackgroundColor AS bgcolor, game.scoreType, game.maxScores, game.thumbnail, game.thumbnailSmall,
			game.help, game.enabled, game.memberGroups,
			IFNULL(score.ID_SCORE,0) AS ID_SCORE, IFNULL(score.score,0) AS champScore, IFNULL(score.startTime, 0) AS champTime,
			IFNULL(mem.ID_MEMBER, 0) AS ID_MEMBER, IFNULL(mem.realName, '') AS realName, IFNULL(score.endTime, 0) AS championTime,
			IFNULL(favorite.ID_FAVORITE, 0) AS isFavorite, IFNULL(category.ID_CAT, 0) AS ID_CAT,
			IFNULL(category.catName, '') As catName, IFNULL(pb.ID_PB, 0) AS ID_PB, IFNULL(pb.score, 0) AS personalBest
		FROM {$db_prefix}arcade_games AS game
			LEFT JOIN {$db_prefix}arcade_scores AS score ON (score.ID_SCORE = game.ID_CHAMPION_SCORE)
			LEFT JOIN {$db_prefix}members AS mem ON (mem.ID_MEMBER = game.ID_MEMBER_CHAMPION)
			LEFT JOIN {$db_prefix}arcade_personalbest AS pb ON (pb.ID_GAME = game.ID_GAME AND pb.ID_MEMBER = $ID_MEMBER)
			LEFT JOIN {$db_prefix}arcade_favorite AS favorite ON (favorite.ID_GAME = game.ID_GAME AND favorite.ID_MEMBER = $ID_MEMBER)
			LEFT JOIN {$db_prefix}arcade_categories AS category ON (category.ID_CAT = game.ID_CAT)
		WHERE $user_info[query_see_game] AND $enabled
		$sql
		LIMIT 0,1";

	$game_result = db_query($query, __FILE__, __LINE__);

	if (mysql_num_rows($game_result) == 0)
		return false; // No such game was found

	$game = mysql_fetch_assoc($game_result);
	mysql_free_result($game_result);

	if (!$admin)
		return BuildGameArray($game);

	else
		return BuildGameArrayAdmin($game);
}

// Updates Game
function UpdateGame($ID_GAME = null, $modifications)
{
	global $scripturl, $txt, $db_prefix, $user_info, $ID_MEMBER;

	if (empty($ID_GAME))
		fatal_error('arcade_game_update_error');
	if (!is_array($ID_GAME))
		$ID_GAME = array($ID_GAME);

	// Following is numeric values
	$numeric = array('gameWidth', 'gameHeight', 'scoreType', 'maxScores', 'numPlays', 'numRates', 'ID_CAT', 'gameRating');
	// Valid values everyone can modify these
	$valid = array('ID_MEMBER_CHAMPION', 'ID_CHAMPION_SCORE', 'numPlays', 'numRates', 'gameRating');
	// Following may not be empty
	$non_empty = array('gameName', 'category', 'internalName', 'gameFile');

	// These can be modfied only by those wo can modify arcade. Just for extra protection
	if (allowedTo('arcade_admin'))
	{
		$valid = array_merge($valid, array('gameDirectory', 'gameType', 'maxScores', 'ID_CAT', 'enabled', 'memberGroups'));

		// What we can update in single game mode
		if (count($ID_GAME) == 1)
			$valid = array_merge($valid, array('gameName', 'description', 'gameFile',
			'internalName', 'gameWidth', 'gameHeight', 'gameBackgroundColor',
			'scoreType', 'thumbnail', 'thumbnailSmall', 'help'));

	}


	$changes = array();
	$errors = array();

	foreach ($modifications as $key => $value)
		if (in_array($key, $valid))
		{
			if (!empty($value) || !in_array($key, $non_empty))
			{
				if (in_array($key, $numeric))
				{
					// Hey is this really numeric?
					if (!is_numeric($value))
						$errors[$key] = 'arcade_must_be_numeric';
					$changes[] = "$key = $value";
				}
				else
				{
					if (is_array($value))
						$value = implode(',', $value);

					$value = mysql_real_escape_string(stripcslashes($value));
					$changes[] = "$key = '$value'";
				}
			}
			else
				$errors[$key] = 'arcade_empty_value';
		}
		// This can't be modifed,
		else
			fatal_error($txt['arcade_game_update_error'] . ' (' . $key . ')');

	// Check some values
	if (isset($modifications['gameBackgroundColor']))
	{
		$modifications['gameBackgroundColor'] = trim($modifications['gameBackgroundColor']);

		if (strlen($modifications['gameBackgroundColor']) != 6 && $modifications['gameBackgroundColor'] != '')
			$errors['gameBackgroundColor'] = 'arcade_invalid_bgcolor';
	}

	if (count($errors) >= 1)
		return $errors;

	if (count($changes) == 0)
		fatal_lang_error('arcade_game_update_error');

	db_query("
		UPDATE {$db_prefix}arcade_games
		SET " . implode(', ', $changes) . "
		WHERE ID_GAME IN(" . implode(', ', $ID_GAME) . ")", __FILE__, __LINE__);

	//if (db_affected_rows() < count($ID_GAME))
	//	fatal_lang_error('arcade_game_update_error');

	// No errors
	return array();
}

function BuildGameArray(&$game)
{
	// Build Game array for easier to use and update

	global $scripturl, $txt, $db_prefix, $modSettings, $user_info, $ID_MEMBER;

	if ($game['gameDirectory'] != '') // Is game installed in subdirectory
		$gameurl = $modSettings['gamesUrl'] . '/' . $game['gameDirectory'] . '/';

	else // It is in main directory
		$gameurl = $modSettings['gamesUrl'] . '/';

	$game['description'] = parse_bbc($game['description']);
	$game['help'] = parse_bbc($game['help']);
	$game['maxScores'] = $game['maxScores'] == - 1 ? (isset($modSettings['arcadeMaxScores']) ? $modSettings['arcadeMaxScores'] : 0 ) : $game['maxScores'];

	return array(
		'id' => $game['ID_GAME'],
		'url' => array(
			'play' => $scripturl . '?action=arcade;sa=play;game=' . $game['ID_GAME'],
			'highscore' => $scripturl . '?action=arcade;sa=highscore;game=' . $game['ID_GAME'],
			'flash' => $gameurl . $game['gameFile'],
			'favorite' => $game['isFavorite'] == 0 ? $scripturl . '?action=arcade;sa=favorite;game=' . $game['ID_GAME'] : $scripturl . '?action=arcade;sa=favorite;remove;game=' . $game['ID_GAME'],
		),
		// Information needed for showing flash
		'flash' => array(
			'width' => $game['gameWidth'],
			'height' => $game['gameHeight'],
			'backgroundColor' => $game['bgcolor']
		),
		'category' => array(
			'id' => $game['ID_CAT'],
			'name' => $game['catName'],
			'link' => $scripturl . '?action=arcade;category=' . $game['ID_CAT'],
		),
		'gameType' => $game['gameType'],
		'internalName' => $game['internalName'],
		'name' => $game['gameName'],
		'file' => $game['gameFile'],
		'description' => $game['description'],
		'help' =>  $game['help'],
		'rating' => $game['gameRating'],
		'rating2' => round($game['gameRating']),
		'thumbnail' => !empty($game['thumbnail']) ? $gameurl . $game['thumbnail'] : '',
		'thumbnailSmall' => !empty($game['thumbnailSmall']) ? $gameurl . $game['thumbnailSmall'] : '',
		'isChampion' => $game['ID_SCORE'] > 0 ? true : false,
		'champion' => array(
			'member_id' => $game['ID_MEMBER'],
			'score_id' => $game['ID_SCORE'],
			'memberLink' =>  $game['realName'] != '' ? '<a href="' . $scripturl . '?action=profile;u=' . $game['ID_MEMBER'] . '">' . $game['realName'] . '</a>' : $txt['arcade_guest'],
			'score' => comma_format($game['champScore']),
			'time' => $game['champTime'],
		),
		'isPersonalBest' => $game['ID_PB'] > 0 ? true : false,
		'personalBest' => comma_format($game['personalBest']),
		'scoreType' => $game['scoreType'],
		'highscoreSupport' => $game['scoreType'] != 2 ? true : false,
		'maxScores' => $game['maxScores'],
		'directory' => $game['gameDirectory'],
		'isFavorite' => $game['isFavorite'] > 0 ? true : false,
	);
}

function BuildGameArrayAdmin($game)
{
	// Build Game array for easier to use and update

	global $scripturl, $txt, $db_prefix, $modSettings, $user_info, $ID_MEMBER;

	return array(
		'id' => $game['ID_GAME'],
		'url' => array(
			'play' => $scripturl . '?action=arcade;sa=play;game=' . $game['ID_GAME'],
			'highscore' => $scripturl . '?action=arcade;sa=highscore;game=' . $game['ID_GAME'],
			'edit' => $scripturl . '?action=managegames;sa=edit;game=' . $game['ID_GAME'],
			'highscore_edit' => $scripturl . '?action=managehighscores;sa=edit;game=' . $game['ID_GAME'],
			'delete' => $scripturl . '?action=managegames;sa=delete;game=' . $game['ID_GAME'],
		),
		// Information needed for showing flash
		'flash' => array(
			'width' => $game['gameWidth'],
			'height' => $game['gameHeight'],
			'backgroundColor' => $game['bgcolor']
		),
		'category' => array(
			'id' => $game['ID_CAT'],
			'name' => $game['catName'],
			'link' => $scripturl . '?action=arcade;category=' . $game['ID_CAT'],
		),
		'gameType' => $game['gameType'],
		'internalName' => $game['internalName'],
		'name' => $game['gameName'],
		'file' => $game['gameFile'],
		'description' => $game['description'],
		'help' =>  $game['help'],
		'rating' => $game['gameRating'],
		'thumbnail' => $game['thumbnail'],
		'thumbnailSmall' => $game['thumbnailSmall'],
		'scoreType' => $game['scoreType'],
		'maxScores' => $game['maxScores'],
		'directory' => $game['gameDirectory'],
		'enabled' => $game['enabled'] == 1,
		'memberGroups' => isset($game['memberGroups']) ? $game['memberGroups'] : '',
	);
}

function ArcadeFlashHtml($game, $auto_start = true, $message = '')
{
	global $txt, $context, $settings;

	if ($auto_start)
		$start = 'arcadeRestart();';
	else
		$start = '';

	return '
	<div id="game">
		' . $message . '
	</div>
	<script type="text/javascript" src="' . $settings['default_theme_url'] . '/swfobject.js"></script>
	<script language="JavaScript" type="text/javascript" defer="defer"><!-- // --><![CDATA[
		var play_url = smf_scripturl + "?action=arcade;sa=play;xml";
		var running = false;

		function arcadeRestart()
		{
			running = false;
			ajax_indicator(true);

			setInnerHTML(document.getElementById("game"), "' . addslashes($txt['arcade_please_wait']) . '");

			var i, x = new Array();

			x[0] = "game=' . $game['id'] . '";
			x[1] = "sesc=' . $context['session_id'] . '";

			sendXMLDocument(play_url, x.join("&"), ArcadeStart);

			return false;
		}

		function ArcadeStart()
		{
			if (running)
				return 0;

			running = true;
			ajax_indicator(false);

			setInnerHTML(document.getElementById("game"), "' . addslashes($txt['arcade_no_flash']) . '");

			var so = new SWFObject("' . $game['url']['flash'] . '", "' . $game['file'] . '", "' . $game['flash']['width'] . '", "' . $game['flash']['height'] . '", "7");
			so.addParam("menu", "false");
			so.write("game");

			return true;
		}

		'. $start .'
	// ]]></script>';

	// #' . $game['flash']['backgroundColor'] . '

}

function ArcadeFixPositions($ID_GAME)
{
	global $db_prefix, $user_info;

	$game = ArcadeGameInfo($ID_GAME);

	if ($game === false)
		return array();

	if ($game['scoreType'] == 0)
		$type = 'DESC';
	elseif ($game['scoreType'] == 1)
		$type = 'ASC';
	else
		return array();

	// Fetch scores
	$scores = db_query("
		SELECT ID_SCORE, position, ID_MEMBER, score
		FROM {$db_prefix}arcade_scores
		WHERE ID_GAME = $ID_GAME
		ORDER BY score $type", __FILE__, __LINE__);

	// Clear personal best list, which will be remade
	db_query("
		DELETE FROM {$db_prefix}arcade_personalbest
		WHERE ID_GAME = $ID_GAME", __FILE__, __LINE__);

	$personal_best = array();

	if (mysql_num_rows($scores) == 0)
	{
		// There is no champion
		updateGame($ID_GAME, array('ID_CHAMPION_SCORE' => 0, 'ID_MEMBER_CHAMPION' => 0));

		return array();
	}

	$position = 0;
	$error = array();

	while ($score = mysql_fetch_assoc($scores))
	{
		$position++;

		if (!isset($personal_best[ $score['ID_MEMBER'] ])) // Set personal best, best will be first so no need to check
			$personal_best[ $score['ID_MEMBER'] ] = array(
				'score' => $score['score'],
				'id' => $score['ID_SCORE'],
			);

		if ($score['position'] != $position)
		{
			$error[] = array(
				'old' => $score['position'],
				'new' => $position,
				'id' => $score['ID_SCORE']
			);

			db_query("
				UPDATE {$db_prefix}arcade_scores
				SET position = $position
				WHERE ID_SCORE = $score[ID_SCORE]", __FILE__, __LINE__);
		}

		if ($position == 1)
			updateGame($ID_GAME, array('ID_CHAMPION_SCORE' => $score['ID_SCORE'], 'ID_MEMBER_CHAMPION' => $score['ID_MEMBER']));
	}
	mysql_free_result($scores);

	foreach ($personal_best as $member => $info)
		db_query("
			INSERT INTO {$db_prefix}arcade_personalbest
			SET ID_GAME = $ID_GAME, ID_MEMBER = $member, score = $info[score]", __FILE__, __LINE__);


	$scores = db_query("
		SELECT ID_SCORE, position, ID_MEMBER, score, endTime
		FROM {$db_prefix}arcade_scores
		WHERE ID_GAME = $ID_GAME
		ORDER BY endTime ASC", __FILE__, __LINE__);

	$maxScore = 0;
	$max_ID_SCORE = 0;

	db_query("
		UPDATE {$db_prefix}arcade_scores
		SET championFrom = 0
		WHERE ID_GAME = $ID_GAME", __FILE__, __LINE__);

	// Update champion times
	while ($score = mysql_fetch_assoc($scores))
	{
		if (($game['scoreType'] == 0 && $maxScore <= $score['score']) || ($game['scoreType'] == 1 && $maxScore >= $score['score']))
		{
			$end = $score['endTime'] - 1;

			if ($max_ID_SCORE > 0)
				db_query("
					UPDATE {$db_prefix}arcade_scores
					SET championFrom = endTime, championTo = $end
					WHERE ID_SCORE = $max_ID_SCORE", __FILE__, __LINE__);

			$maxScore = $score['score'];
			$max_ID_SCORE = $score['ID_SCORE'];
		}
	}
	mysql_free_result($scores);

	db_query("
		UPDATE {$db_prefix}arcade_scores
		SET championFrom = endTime, championTo = 0
		WHERE ID_SCORE = $max_ID_SCORE", __FILE__, __LINE__);

	return $error;
}

function ArcadeSaveScore($game, $score, $startTime, $endTime, $scoreStatus)
{
	global $ID_MEMBER, $user_info, $db_prefix, $modSettings, $context;

	if (!$context['arcade']['can_submit'])
		return false;

	if ($game['scoreType'] == 0)
		$reverse = false;

	elseif ($game['scoreType'] == 1)
		$reverse = true;

	else
	{
		if ($user_info['is_admin'])
			fatal_error('Game does not support scoring.', false);
		else
			return false; // Cheater tries to add score to game which doesn't even support, suspicous?
	}

	$ID_GAME = $game['id'];

	$duration = $endTime - $startTime;
	$ip = $user_info['ip'];

	// Reverse must compares reversed
	$better = $reverse ? '<' : '>';
	$worse = !$reverse ? '<' : '>';

	// Get position
	$posres = db_query("
		SELECT count(*) AS pos
		FROM {$db_prefix}arcade_scores
		WHERE score $better $score AND ID_GAME = $ID_GAME", __FILE__, __LINE__);

	$pos_res = mysql_fetch_assoc($posres);
	$pos = $pos_res['pos'] + 1;
	mysql_free_result($posres);

	// If theres max socres limit  then let's check whatever to save or not to save
	if ($game['maxScores'] >= 1)
	{

		$my_results = db_query("
			SELECT count(*) AS count
			FROM {$db_prefix}arcade_scores
			WHERE ID_GAME = $ID_GAME AND ID_MEMBER = $ID_MEMBER", __FILE__, __LINE__);

		$my = mysql_fetch_assoc($my_results);
		mysql_free_result($my_results);

		if ($my['count'] < $game['maxScores'])
			$save = true;

		else
		{
			// Clear out worst scores to maintain maximum number of scores
			// so if it has been changed it will update now

			$to_remove = $my['count'] - $game['maxScores'] + 1;

			$remove = db_query("
				SELECT position, score, ID_SCORE
				FROM {$db_prefix}arcade_scores
				WHERE ID_GAME = $ID_GAME AND ID_MEMBER = $ID_MEMBER
				ORDER BY position DESC", __FILE__, __LINE__);

			$remove_id = array();

			// Let's check which scores should be removed
			while ($myscore = mysql_fetch_assoc($remove))
				if (($myscore['position'] > $pos && $to_remove > 0) ||
					($myscore['position'] == $pos && $to_remove > 0 &&
					 (($score > $myscore['score'] && !$reverse) ||
					  ($score < $myscore['score'] && $reverse)
					 )
					) || $to_remove > 1)
				{
					$remove_id[] = $myscore['ID_SCORE'];
					$to_remove--;
				}

			mysql_free_result($remove);

			$removed = count($remove_id);

			// Were there scores to be removed?
			if ($removed > 0)
				db_query("
					DELETE FROM {$db_prefix}arcade_scores
					WHERE ID_SCORE IN(" . implode(',', $remove_id) . ")", __FIlE__, __LINE__);

			// Can we add more scores?
			if (($my['count'] - $removed) < $game['maxScores'])
				$save = true;

			else
			{
				$save = false;
				$error = 'arcade_scores_limit';
			}
		}
	}
	else
		$save = true;


	// Can't save, sorry!
	if (!$save)
		return array(
			'ID_SCORE' => false,
			'error' => $error
		);

	if ($pos == 1)
		$championFrom = $endTime;

	else
		$championFrom = 0;

	db_query("
		INSERT INTO {$db_prefix}arcade_scores
		SET
			ID_GAME = $ID_GAME, ID_MEMBER = $ID_MEMBER, score = $score, startTime = $startTime,
			endTime = $endTime, gameDuration = $duration, memberIp = '$ip', scoreStatus = $scoreStatus, comment = '',
			position = $pos, championFrom = $championFrom, championTo = 0", __FILE__, __LINE__);

	$ID_SCORE = db_insert_id();
	$ownbest = false;

	if ($pos == 1)
	{
		// You are now the champion! (or theres serious error in database which should be handled by maintenance functions)
		updateGame($ID_GAME, array('ID_CHAMPION_SCORE' => $ID_SCORE, 'ID_MEMBER_CHAMPION' => $ID_MEMBER));

		if ($game['isChampion'])
		{
			db_query("
				UPDATE {$db_prefix}arcade_scores
				SET championTo = $endTime
				WHERE ID_SCORE =  {$game['champion']['score_id']}", __FILE__, __LINE__);

			arcadesendPM('new_champion', $game['champion']['member_id'], $game, array('score' => $score,'time' => time()));
		}
	}

	// Lets update personal best
	$isres = db_query("
		SELECT score
		FROM {$db_prefix}arcade_personalbest
		WHERE ID_GAME = $ID_GAME AND ID_MEMBER = $ID_MEMBER", __FILE__, __LINE__);

	if (mysql_num_rows($isres) == 0)
	{
		// No Personal best? Then we just insert this score
		db_query("
			INSERT INTO {$db_prefix}arcade_personalbest
			SET ID_GAME = $ID_GAME, ID_MEMBER = $ID_MEMBER, score = $score", __FILE__, __LINE__);

		$ownbest = true;
	}
	else
	{
		$scorex = mysql_fetch_assoc($isres);

		// Update score if its better :>
		if (!$reverse && $scorex['score'] < $score)
		{
			db_query("
				UPDATE {$db_prefix}arcade_personalbest
				SET score = $score
				WHERE ID_GAME = $ID_GAME AND ID_MEMBER = $ID_MEMBER", __FILE__, __LINE__);

			$ownbest = true;
		}
		elseif ($reverse && $scorex['score'] > $score)
		{
				db_query("
					UPDATE {$db_prefix}arcade_personalbest
					SET score = $score
					WHERE ID_GAME = $ID_GAME AND ID_MEMBER = $ID_MEMBER", __FILE__, __LINE__);

				$ownbest = true;
		}
	}

	mysql_free_result($isres);

	// Update positions
	$result = db_query("
		SELECT ID_SCORE
		FROM {$db_prefix}arcade_scores
		WHERE ID_GAME = $ID_GAME AND position >= $pos AND ID_SCORE != $ID_SCORE
		ORDER BY position ASC", __FILE__, __LINE__);

	$new_pos = $pos + 1;
	while ($score = mysql_fetch_assoc($result))
	{
		db_query("
			UPDATE {$db_prefix}arcade_scores
			SET position = $new_pos
			WHERE ID_SCORE = $score[ID_SCORE]", __FILE__, __LINE__);

		$new_pos++;
	}

	// Where should we start?
	$scoresPerPage = isset($modSettings['scoresPerPage']) ? $modSettings['scoresPerPage'] : 50;
	$start = (floor($pos / $scoresPerPage)) * $scoresPerPage;

	// Return array with ID_SCORE and later maybe something more
	return array(
		'ID_SCORE' => $ID_SCORE,
		'position' => $pos,
		'new_champion' => $pos == 1 ? true : false,
		'start' => $start,
		'ownbest' => $ownbest,
	);
}

function ArcadeSendPM($type, $to, $game, $extra = null)
{
	global $db_prefix, $sourcedir, $txt, $user_info, $ID_MEMBER;

	if ($to == $ID_MEMBER)
		return false;

	loadLanguage('PersonalMessage');

	if ($type == 'new_champion')
	{
		// Some beat your

		require_once($sourcedir . '/Subs-Post.php');

		$subject = sprintf($txt['arcade_pm_champion_beat_subject'], $game['name']);
		$message = $txt['arcade_pm_automatic_notification'] . ' ' . sprintf($txt['arcade_pm_champion_beat_mesage'], $user_info['name'], '[iurl=' . $game['url']['play'] . ']' . $game['name'] . '[/iurl]', $game['url']['play']);

		return sendpm(array('to' => array($to), 'bcc' => array()), addslashes($subject), addslashes($message));
	}

	return false;
}

function &prepareCategories()
{
	global $db_prefix;

	static $category = array(), $run = false;

	if ($run)
		return $category;

	$categories_result = db_query("
		SELECT ID_CAT AS id, catName AS name, numGames AS cntGame, memberGroups, special, catOrder
		FROM {$db_prefix}arcade_categories
		ORDER BY catOrder", __FILE__, __LINE__);

	// Add categories to array
	while ($cat = mysql_fetch_array($categories_result))
		$category[ $cat['id'] ] = array(
			'id' => $cat['id'],
			'name' => $cat['name'],
			'games' => $cat['cntGame'],
			'default' => $cat['special'] == 1 ? true : false,
			'canRemove' => $cat['special'] == 1 ? false : true,
			'memberGroups' => explode(',', $cat['memberGroups']),
		);

	mysql_free_result($categories_result);

	$run = true;

	return $category;
}

function FixCategories()
{
	global $db_prefix, $modSettings;

	if (!isset($modSettings['arcadeDefaultCategory']))
		fatal_lang_error('arcade_category_no_default');

	$res = db_query("
		SELECT catName
		FROM {$db_prefix}arcade_categories
		WHERE ID_CAT = $modSettings[arcadeDefaultCategory] AND special = 1", __FILE__, __LINE__);

	if (mysql_num_rows($res) == 0)
		fatal_lang_error('arcade_category_no_default');

	$res = db_query("
		SELECT ID_GAME, IFNULL(cat.catName, '') as cn
		FROM {$db_prefix}arcade_games AS game
		LEFT JOIN {$db_prefix}arcade_categories AS cat ON cat.ID_CAT = game.ID_CAT", __FILE__, __LINE__);

	while ($game = mysql_fetch_assoc($res))
		if ($game['cn'] == '')
			db_query("
				UPDATE {$db_prefix}arcade_games
				SET ID_CAT = $modSettings[arcadeDefaultCategory]
				WHERE ID_GAME = $game[ID_GAME]", __FILE__, __LINE__);

	redirectexit('action=arcadeadmin;maintenace=done');
}

function prepareMemberGroups($ryhmat = null)
{
	global $db_prefix, $modSettings, $context, $txt;

	if ($ryhmat == null)
		$ryhmat = array();

	// Load membergroups.
	$request = db_query("
		SELECT groupName, ID_GROUP, minPosts
		FROM {$db_prefix}membergroups
		WHERE ID_GROUP > 3 OR ID_GROUP = 2
		ORDER BY minPosts, ID_GROUP != 2, groupName", __FILE__, __LINE__);

	$context['groups'][-1] = array(
		'id' => -1,
		'name' => $txt['membergroups_guests'],
		'checked' => in_array(-1, $ryhmat),
		'is_post_group' => false,
	);

	$context['groups'][0] = array(
		'id' => 0,
		'name' => $txt['membergroups_members'],
		'checked' => in_array(0, $ryhmat),
		'is_post_group' => false,
	);

	while ($row = mysql_fetch_assoc($request))
		$context['groups'][(int) $row['ID_GROUP']] = array(
			'id' => $row['ID_GROUP'],
			'name' => trim($row['groupName']),
			'checked' => in_array($row['ID_GROUP'], $ryhmat),
			'is_post_group' => $row['minPosts'] != -1,
	);


	mysql_free_result($request);
}

function microtime_float()
{
   list($usec, $sec) = explode(" ", microtime());
   return (float) $usec + (float) $sec;
}

?>